/**
 * OW2 Util
 * Copyright (C) 2010-2012 Bull S.A.S.
 * Copyright (C) 2008 SERLI
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id PlanDeployableActivator.java $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.plan.deploy.deployable;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.ow2.util.ee.deploy.impl.helper.DeployableHelper;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.plan.deploy.deployable.api.factory.IFileDeployableFactoryManager;
import org.ow2.util.plan.deploy.deployable.api.factory.XmlFileDeployableFactory;
import org.ow2.util.plan.deploy.deployable.api.profile.IDeploymentProfileManager;
import org.ow2.util.plan.deploy.deployable.impl.DeploymentPlanDeployableImpl;
import org.ow2.util.plan.deploy.deployable.impl.RepositoryDeployableImpl;
import org.ow2.util.plan.deploy.deployable.impl.factory.DefaultFileDeployableFactoryManager;
import org.ow2.util.plan.deploy.deployable.impl.factory.XmlFileDeployableFactoryImpl;
import org.ow2.util.plan.deploy.deployable.impl.profile.DeploymentProfileManagerImpl;


/**
 * Activator for the deploy bundle.
 * @author Mickaël LEDUQUE
 */
public class PlanDeployableActivator implements BundleActivator  {

    /**
     * The logger.
     */
    private Log logger = LogFactory.getLog(PlanDeployableActivator.class);

    /**
     * Deployment plan namespace.
     */
    private static final String DEPLOYMENTPLAN_NS =
            "http://jonas.ow2.org/ns/deployment-plan/1.0";

    /**
     * repositories namespace.
     */
    private static final String REPOSITORY_NS =
            "http://jonas.ow2.org/ns/deployment-plan/repositories/1.0";

    /**
     * The ServiceRegistration object for the registered IFileDeployableFactoryManager object.
     */
    private ServiceRegistration factoryManagerServiceRegistration = null;

    /**
     * The ServiceRegistration object for the registered XmlFileDeployableFactory object.
     */
    private ServiceRegistration fileDeployableFactoryServiceRegistration;

    private ServiceRegistration deploymentProfileManagerServiceRegistration;

    /**
     * {@inheritDoc}
     */
    public void start(BundleContext context) throws Exception {
        /* Will register a DefaultFileDeployableFactoryManager
         * and add an XmlFileDeployableFactory with deployment plan
         * and repository deployable classes registered */
        IFileDeployableFactoryManager fileDeployableFactoryManager =
                new DefaultFileDeployableFactoryManager();
        factoryManagerServiceRegistration = context.registerService(
                IFileDeployableFactoryManager.class.getName(),
                fileDeployableFactoryManager, null);

        IDeploymentProfileManager deploymentProfileManager = new DeploymentProfileManagerImpl();
        deploymentProfileManagerServiceRegistration = context.registerService(
                IDeploymentProfileManager.class.getName(), deploymentProfileManager, null);

        XmlFileDeployableFactory fileDeployableFactory =
                new XmlFileDeployableFactoryImpl();
        fileDeployableFactoryServiceRegistration = context.registerService(
                XmlFileDeployableFactory.class.getName(),
                fileDeployableFactory, null);
        fileDeployableFactory.registerFileDeployable(
                DeploymentPlanDeployableImpl.class, DEPLOYMENTPLAN_NS);
        fileDeployableFactory.registerFileDeployable(
                RepositoryDeployableImpl.class, REPOSITORY_NS);
        fileDeployableFactoryManager.addFileDeployableFactory(fileDeployableFactory);

        /* Unfortunately, DeployableHelper is a mainly static class */
        DeployableHelper.setFileDeployableFactory(fileDeployableFactoryManager);
        logger.debug("FileDeployableFactory service registered and used in DeployableHelper");
    }

    /**
     * {@inheritDoc}
     */
    public void stop(BundleContext context) throws Exception {
        DeployableHelper.setFileDeployableFactory(null);
        fileDeployableFactoryServiceRegistration.unregister();
        deploymentProfileManagerServiceRegistration.unregister();
        factoryManagerServiceRegistration.unregister();
        logger.debug("FileDeployableFactory service unregistered");
    }

}
